import * as Realm from "realm";
import { Discovery } from '../discovery/Discovery';
import { Server } from "../Server";
import { Logger } from "../shared/Logger";
import { IAuthProvider } from "../auth";
export interface AuthenticationResponseJSON {
    refresh_token: {
        token: string;
        token_data: any;
    };
}
export declare class User {
    static schema: {
        name: string;
        primaryKey: string;
        properties: {
            userId: {
                type: string;
                optional: boolean;
            };
            isAdmin: {
                type: string;
                optional: boolean;
                default: boolean;
            };
            accounts: {
                type: string;
                objectType: string;
            };
            metadata: {
                type: string;
                objectType: string;
            };
        };
    };
    userId: string;
    isAdmin: boolean;
    accounts?: Account[];
    metadata?: Realm.List<UserMetadataRow>;
    created?: boolean;
    toJSON(): {
        user_id: string;
        is_admin: boolean;
        accounts: {
            provider: string;
            provider_id: string;
        }[];
        metadata: UserMetadataRow[];
    };
}
export declare class Account {
    static schema: Realm.ObjectSchema;
    provider: string;
    providerId: string;
    user?: User;
    toJSON(): {
        provider: string;
        provider_id: string;
    };
}
export declare type UserMetadataRow = {
    user?: User;
    key: string;
    value: string;
};
export declare class Metadata {
    static schema: Realm.ObjectSchema;
}
export declare class TokenRevocation {
    static schema: Realm.ObjectSchema;
}
export interface AuthServiceConfiguration {
    refreshTokenTtl?: number;
}
export declare class AuthService {
    logger: Logger;
    privateKey: string;
    publicKey: string;
    protected refreshTokenTtl: number;
    constructor(config?: AuthServiceConfiguration);
    private syncedAdminRealm;
    readonly adminRealm: Realm;
    private syncedRevocationRealm;
    readonly revocationRealm: Realm;
    discovery: Discovery;
    dataPath: string;
    providers: IAuthProvider[];
    addProvider(provider: IAuthProvider): void;
    protected cleanupInterval: number;
    start(server: Server): Promise<void>;
    stop(): Promise<void>;
    authenticate(request: any): Promise<AuthenticationResponseJSON>;
    revokeToken(tokenData: string, req?: any): Promise<{}>;
    getUserById(userId: string, req?: any): User;
    getUserByProviderId(provider: string, providerId: string, req?: any): User;
    createOrUpdateUser(providerId: string, provider: string, isAdmin: boolean, metadata?: {
        [key: string]: string;
    }, userId?: string, req?: any): Promise<User>;
    updateProviderData(providerName: string, userId: string, data: any, req?: any): Promise<{}>;
    private pruneRevocationTokens();
    private generateUniqueId();
    private checkNotRevoked(token);
    private requireAdminOrUserId(req?, userId?);
}
