/// <reference types="express" />
import { Logger } from '../shared/Logger';
import { Server } from "../Server";
import * as Realm from 'realm';
import { Request } from 'express';
export interface RealmAccessTokenResponse {
    access_token: {
        token: string;
        token_data: any;
    };
}
export interface RealmDirectoryServiceConfig {
    accessTokenTtl?: number;
    tags?: string[];
}
export declare class RealmDirectoryService {
    protected logger: Logger;
    protected privateKey: string;
    protected publicKey: string;
    tags: string[];
    config: RealmDirectoryServiceConfig;
    private discovery;
    private syncedRealm;
    private accessTokenTtl;
    readonly realm: Realm;
    constructor(config?: RealmDirectoryServiceConfig);
    private accessLevelForPath(path, userId);
    setLogger(l: Logger): void;
    accessToken(path: string, data: any, app_id?: string): Promise<RealmAccessTokenResponse>;
    find(): Promise<{
        path: string;
        creatorId: string;
        creationDate: Date;
        permissions: Realm.List<{
            path: string;
            accessLevel: number;
            type: number;
            key: string;
            value: string;
        }>;
    }[]>;
    findByPath(realmPath: string, accessToken: string): Promise<{
        path: string;
        creatorId: string;
        creationDate: Date;
        permissions: Realm.List<{
            path: string;
            accessLevel: number;
            type: number;
            key: string;
            value: string;
        }>;
    }>;
    private validatedRealmFile(path);
    private realmFileJSON(realmFile, withPermissions);
    get(incomingRequest: Request): Promise<{
        path: string;
        creatorId: string;
        creationDate: Date;
        permissions?: {
            key: string;
            value: string;
            type: "user" | "all" | "metadata_exact" | "metadata_regex";
            accessLevel: Realm.Sync.AccessLevel;
        }[];
    }>;
    remove(incomingRequest: Request): Promise<void>;
    getPermissions(incomingRequest: Request): Promise<{
        key: string;
        value: string;
        type: "user" | "all" | "metadata_exact" | "metadata_regex";
        accessLevel: Realm.Sync.AccessLevel;
    }[]>;
    createPermissions(incomingRequest: Request): Promise<{}>;
    start(server: Server): Promise<void>;
    stop(): Promise<void>;
    private createRealmFile(path, creatorId, syncLabel?);
}
