"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const decorators_1 = require("../decorators");
const Logger_1 = require("../shared/Logger");
const Server_1 = require("../Server");
const errors = require("../errors");
const Token_1 = require("../shared/Token");
const httpProxy = require("http-proxy");
const superagent = require("superagent");
const URI = require("urijs");
let SyncProxyService = class SyncProxyService {
    start(server) {
        return __awaiter(this, void 0, void 0, function* () {
            this.discovery = server.discovery;
            this.publicKey = server.publicKey;
        });
    }
    setLogger(logger) {
        this.logger = logger;
    }
    websocketHandler(req, socket, head) {
        return __awaiter(this, void 0, void 0, function* () {
            const authorization = req.headers['authorization'];
            if (!authorization) {
                throw new errors.JSONError({
                    title: "Authorization header was not provided",
                    status: 400,
                });
            }
            const matches = authorization.match(/^Realm\-Access\-Token version=(\d+) token="(.*)"$/);
            if (!matches) {
                throw new errors.JSONError({
                    title: "Authorization header is not in a valid format",
                    status: 400,
                });
            }
            const tokenVersion = matches[1];
            const tokenData = matches[2];
            const pathParam = req.params.path;
            if (!pathParam) {
                throw new errors.realm.MissingParameters('path');
            }
            const encodedPath = encodeURIComponent(pathParam);
            let token;
            try {
                token = Token_1.Token.parse(tokenData, this.publicKey);
            }
            catch (err) {
                throw new errors.realm.InvalidCredentials();
            }
            if (!(token instanceof Token_1.AccessToken)) {
                throw new errors.realm.InvalidParameters();
            }
            if (token.path !== pathParam) {
                throw new errors.realm.InvalidParameters();
            }
            let label = token.syncLabel;
            if (!label || label === "") {
                label = yield this.findSyncLabelForPath(pathParam, tokenData);
                if (!label) {
                    throw new errors.JSONError({
                        title: "AccessToken is not valid: sync label was not found in token",
                        status: 400,
                    });
                }
            }
            const syncHandle = yield this.discovery.find("sync", ["master", label]);
            if (!syncHandle) {
                throw new errors.JSONError({
                    title: "Backend sync service is unavailable",
                    status: 503,
                });
            }
            const target = `http://${syncHandle.address}:${syncHandle.port}`;
            const proxy = httpProxy.createProxy({ ws: true, target });
            proxy.on('error', (err) => {
                this.logger.warn("sync proxy error", err);
            });
            proxy.on('close', () => {
                this.logger.debug("sync proxy closed");
            });
            proxy.ws(req, socket, head);
        });
    }
    findSyncLabelForPath(path, tokenData) {
        return __awaiter(this, void 0, void 0, function* () {
            const realmsServiceHandle = yield this.discovery.find('realms');
            if (!realmsServiceHandle) {
                throw new errors.JSONError({
                    title: "Backend service is unavailable",
                    status: 503,
                });
            }
            const realmsServiceUrl = new URI(`http://${realmsServiceHandle.address}:${realmsServiceHandle.port}`)
                .segment("/realms/files/")
                .segment(`/${path}`)
                .toString();
            try {
                const response = yield superagent.get(realmsServiceUrl)
                    .set({
                    'X-Realm-Access-Token': tokenData,
                });
                return response.body.syncLabel;
            }
            catch (err) {
                return;
            }
        });
    }
};
__decorate([
    decorators_1.Start(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Server_1.Server]),
    __metadata("design:returntype", Promise)
], SyncProxyService.prototype, "start", null);
__decorate([
    decorators_1.Unmute(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Logger_1.Logger]),
    __metadata("design:returntype", void 0)
], SyncProxyService.prototype, "setLogger", null);
__decorate([
    decorators_1.Upgrade('/:path'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], SyncProxyService.prototype, "websocketHandler", null);
SyncProxyService = __decorate([
    decorators_1.BaseRoute('/realm-sync')
], SyncProxyService);
exports.SyncProxyService = SyncProxyService;
//# sourceMappingURL=SyncProxyService.js.map